//
// Gnuplot interface widget
// Copyrigth Jacques Tremblay jackt@gel.ulaval.ca
//
//
// This widget is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//

#include <stdio.h>
#include <fstream.h>
#include <unistd.h>

#include "Fl/fl_draw.h"
#include "Fl_Gnuplot.h"

#define N_ENTRIES 8192

Fl_Gnuplot::Fl_Gnuplot(int x, int y, int w, int h, const char *l) :
  Fl_Box(x,y,w,h,l) {
  filename[0] = 0;
  loaded = 0;
  scaleX = 1.0;
  scaleY = 1.0;

	currentIndex=0;

  nEntries = 0;
  nMaxEntries = N_ENTRIES;
  entries = new OutputEntry[nMaxEntries];

  tmpnam(pre);
  tmpnam(post);
  tmpnam(data);
  tmpnam(script);

  ofstream f1(pre);
  if (f1) {
    f1 << "set term xlib\n";
  }
  ofstream f2(post);
  ofstream f3(data);
	ofstream f4(script);

	set_filename(script);
}

Fl_Gnuplot::~Fl_Gnuplot() {
  unlink(pre);
  unlink(post);
  unlink(data);
  unlink(script);
	delete [] entries;
}

void
Fl_Gnuplot::clear_entries(int release) {
  nEntries = 0;
  if (!release || nMaxEntries <= N_ENTRIES) 
    return;
	cerr << "clear_entries\n";
  delete [] entries;
  nMaxEntries = N_ENTRIES;
  entries = new OutputEntry[nMaxEntries]; 
	cerr << "/clear_entries\n";
}

void
Fl_Gnuplot::add_entry(char *buf) {
  if (nEntries == nMaxEntries) {
    nMaxEntries += N_ENTRIES;
    OutputEntry *nn = new OutputEntry[nMaxEntries];
    int i;
    for(i=0 ; i<nEntries ; i++) {
      nn[i] = entries[i];
    }
    delete [] entries;
    entries = nn;
  }

	if (buf[0] == 'G') {
    clear_entries();
    nEntries = 1;
    entries[0].k = 'G';
		delete [] entries[0].t;
		entries[0].t = 0;
		return;
	}

  OutputEntry &e = entries[nEntries];
  nEntries++;
	
  char *str;
  e.k = buf[0];
	delete [] e.t;
  e.t = 0;

  switch(e.k) {
  case 'L':
    sscanf(buf, "L%4d", &e.x);
    e.x = (e.x % 8) + 2;
    break;
  case 'M':
    sscanf(buf, "M%4d%4d", &e.x, &e.y);
    break;
  case 'V':
    sscanf(buf, "V%4d%4d", &e.x, &e.y);
    break;
  case 'J':
    sscanf(buf, "J%4d", &e.x);
    break;
  case 'T':
    sscanf(buf, "T%4d%4d", &e.x, &e.y);
    str = buf + 9;
    e.t = new char[strlen(str)+1];
    strcpy(e.t, str);
    break;
    break;
  case 'R':
    break;
  }
  
}


enum JUSTIFY {
	LEFT, CENTRE, RIGHT
};

static int _colors[13] = { FL_WHITE, FL_BLACK, FL_BLACK, FL_BLACK,
			   FL_BLACK, FL_RED, FL_GREEN, FL_BLUE,
			   FL_MAGENTA, FL_CYAN, FL_GRAY, FL_GRAY, FL_GRAY };

void
Fl_Gnuplot::load() {
  clear_entries(1);

  scaleX = 1/4096.0*w();
  scaleY = 1/4096.0*h();

  if (!filename[0])
    return;

  FILE *p;
  char buf[10*MAXPATHLEN];

  build_command(buf);
  p = popen(buf, "r");
  if (!p) {
    printf("could not open pipe\n");
    return;
  }

  while(!feof(p)) {
    if (fgets(buf, 1023, p)) {
      add_entry(buf);
    }
  }
  pclose(p);
  loaded = 1;
}

void
Fl_Gnuplot::build_command(char *buf) {
  sprintf(buf, "gnuplot %s %s %s", pre, filename, post);
//  sprintf(buf, "/opt/gnuplot-3.7/gnuplot %s %s %s", pre, filename, post);

}

void
Fl_Gnuplot::play() const {
  char *str;
  double sw;
  int jmode;
  int cx, cy;
  int x1, y1;
  int lt;
  int i;

  fl_font(FL_TIMES, 12);
  fl_color(FL_BLACK);

  for(i=0 ; i<nEntries ; i++) { 
    OutputEntry &e = entries[i];
    switch(e.k) {
    case 'G':
      draw_box();
      draw_label();
      break;
    case 'L':
      lt = e.x;
      fl_color(_colors[lt+3]);
      break;
    case 'M':
      cx = e.x; cy = e.y;
      break;
    case 'V':
      x1 = e.x; y1 = e.y;
      fl_line(x() + x1*scaleX, y() + h()-y1*scaleY, x() + cx*scaleX, y() + h()-cy*scaleY);
      cx = x1;
      cy = y1;
      break;
    case 'J':
      jmode = e.x;
      break;
    case 'T':
      x1 = e.x; y1 = e.y;
      str = e.t;
      sw = fl_width(str);
      switch (jmode) {
      case LEFT:
	sw = 0;
	break;
      case CENTRE:
	sw = -sw / 2;
	break;
      case RIGHT:
	sw = -sw;
	break;
      }
      fl_color(_colors[2]);
      fl_draw(str, x() + x1*scaleX + sw, y() + h()-y1*scaleY + fl_height()/3);	   
      fl_color(_colors[lt+3]);
      break;
    case 'R': 
      break;
    }
  }
}

void
Fl_Gnuplot::draw() {
  if (!loaded)
    load();
  play();
}

void
Fl_Gnuplot::set_2d_data(const double *xx, const double *yy, 
												int n, const char *title) {		
	ofstream scriptFile(script);
	ofstream dataFile(data);
		
	int i;
		
	currentIndex = 0;

	if (!title || !*title)
		scriptFile << "set nokey\n";
	scriptFile << "plot '" << data << "' index 0 ";
	if (title && *title)
		scriptFile << "title '" << title << "' ";
	scriptFile << " with lines\n";

	for(i=0 ; i<n ; i++) {
		dataFile << xx[i] << " " << yy[i] << endl;
	}
	set_filename(script);
}

void
Fl_Gnuplot::add_2d_data(const double *xx, const double *yy, 
												int n, const char *title) {
	fstream scriptFile(script, ios::app);
	fstream dataFile(data, ios::app);
		
	int i;

	currentIndex++;

	if (!title || !*title)
		scriptFile << "set nokey\n";
	scriptFile << "replot '" << data << "' index " << currentIndex;
	if (title && *title)
		scriptFile << " title '" << title << "' ";
	scriptFile << " with lines\n";

	dataFile << "\n\n";
	for(i=0 ; i<n ; i++) {
		dataFile << xx[i] << " " << yy[i] << endl;
	}
	set_filename(script);
}

void
Fl_Gnuplot::set_2d_polar_data(const double *xx, const double *yy, 
															int n, const char *title) {
	ofstream scriptFile(script);
	ofstream dataFile(data);
		
	int i;
		
	currentIndex = 0;

	scriptFile << "set polar\n";

	if (!title || !*title)
		scriptFile << "set nokey\n";
	scriptFile << "plot '" << data << "' index 0 ";
	if (title && *title)
		scriptFile << "title '" << title << "' ";
	scriptFile << " with lines\n";

	for(i=0 ; i<n ; i++) {
		dataFile << xx[i] << " " << yy[i] << endl;
	}
	set_filename(script);
}

void
Fl_Gnuplot::add_2d_polar_data(const double *xx, const double *yy, 
															int n, const char *title) {
	fstream scriptFile(script, ios::app);
	fstream dataFile(data, ios::app);
	
	int i;

	currentIndex++;

	if (!title || !*title)
		scriptFile << "set nokey\n";
	scriptFile << "replot '" << data << "' index " << currentIndex;
	if (title && *title)
		scriptFile << " title '" << title << "' ";
	scriptFile << " with lines\n";

	dataFile << "\n\n";
	for(i=0 ; i<n ; i++) {
		dataFile << xx[i] << " " << yy[i] << endl;
	}
	set_filename(script);
}

void
Fl_Gnuplot::resize(int _x, int _y, int _w, int _h) {
	Fl_Box::resize(_x, _y, _w, _h);
  scaleX = 1/4096.0*w();
  scaleY = 1/4096.0*h();
}

