//
// Gnuplot interface widget
// Copyrigth Jacques Tremblay jackt@gel.ulaval.ca
//
//
// This widget is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//

#ifndef _FL_GNUPLOT_HEADER_
#define _FL_GNUPLOT_HEADER_

#include <sys/param.h>
#include <string.h>
#include "Fl/Fl_Widget.h"
#include "Fl/Fl_Box.h"

class Fl_Gnuplot : public Fl_Box {
private:
  Fl_Gnuplot();
protected:
  char pre[MAXPATHLEN];
  char post[MAXPATHLEN];
  char data[MAXPATHLEN];
  char script[MAXPATHLEN];
	int currentIndex;

  char filename[MAXPATHLEN];
  int loaded;

  void play() const;
  void load();

  double scaleX, scaleY;

  struct OutputEntry {
    char k;
    int x, y;
    char *t;
    
    OutputEntry() : k('G'), x(0), y(0), t(0) {}
		OutputEntry &operator=(OutputEntry &e) {
			delete [] t;
			k = e.k; x = e.x; y = e.y; 
			t = e.t;
			e.t = 0;
		}
    ~OutputEntry() { delete [] t;}
  private:
    OutputEntry(const OutputEntry &e);
		OutputEntry &operator=(const OutputEntry &e);
  };

  OutputEntry *entries;
  int nEntries;
  int nMaxEntries;
  void clear_entries(int release=0);
  void add_entry(char *);
  
  virtual void build_command(char *);

public:

	const char *pre_filename() const { return pre; }
	const char *post_filename() const { return post; }
	const char *script_filename() const { return script; }
	const char *data_filename() const { return data; }
	void touch() { loaded=0; }

  Fl_Gnuplot(int x, int y, int w, int h, const char *l=0);
  ~Fl_Gnuplot();
    // **ERROR**
    //set_filename(const char *f) {
    //ANSI C++ forbids declaration `set_filename' with no type
    //
    void set_filename(const char *f) {
    strcpy(filename,(const char *) f);
    loaded = 0;
  }

  void draw();
	void resize(int x, int y, int w, int h);

	void set_2d_data(const double *, const double *, int n, const char *label = 0);
	void add_2d_data(const double *, const double *, int n, const char *label = 0);
	
	void set_2d_polar_data(const double *, const double *, int n, const char *label = 0);
	void add_2d_polar_data(const double *, const double *, int n, const char *label = 0);
};

#endif // _FL_GNUPLOT_HEADER_
